/******************************************************************************
 * EventContainer.hpp                                                         *
 *                                                                            *
 * Read in variables from EventTree and put them into an Object               *
 *                                                                            *
 * Public Members of EventContainer    class                                  *
 *    EventContainer()                  -- Default Constructor                *
 *    ~EventContainer()                 -- Destructor                         *
 *    GetSourceName()                   -- Get name of source being run on    *
 *    GetSourceNumber()                 -- Get number of source being run on  *
 *    GetTargetTopMass()                -- Get Top Quark Mass                 *
 *    SetSourceName()                   -- Set name of source being run on    *
 *    SetSourceNumber()                 -- Set number of source being run on  *
 *    SetTargetTopMass()                -- Set Top Quark Mass                 *
 *    Initialize()                      -- Initialize class                   *
 *    MakeTopQuarks()                   -- Reconstruct Top in several ways    *
 *    GetNextEvent()                    -- Set up particles for next event    *
 *    SetEventCount()                   -- Set counter for specific event     *
 *    SetLeptonSelect()                 -- Only process lep of certain flavor *
 *    GetLeptonSelect()                 -- Only process lep of certain flavor *
 *    SetNormalization()                -- Set norm factor for all histograms *
 *    GetCollectionTree()               -- Get the collection tree            *
 *    GetEventTree()                    -- Get the event tree                 *
 *    GetTruthTree()                    -- Get the truth tree                 *
 *    GetDebugLevel()                   -- Get debug level                    *
 *    SetDebugLevel()                   -- Set debug level                    *
 *    GetEventWeight()                  -- Get Event Weight                   *
 *    GetOutputEventWeight()            -- Get Output EventWeight             *
 *    GetTreeEventWeight()              -- Get Tree Event Weight              *
 *    GetGlobalEventWeight()            -- Get Global EventWeight             *
 *    SetGlobalEventWeight()            -- Set global event weight            *
 *    SetOutputEventWeight()            -- Set output event weight            *
 *    GetConfig()                       -- Get Configuration                  *
 *                                                                            *
 *    Int_t ReadEvent()                 -- Create TLorentz Vectors            *
 *    Bool_t CreateFinalState()         -- Create final state particles       *
 *    Int_t runNumber                   -- Run number from the root tree      *
 *    Int_t eventNumber                 -- Event number from the root tree    *
 *    vector<Electron> electrons        -- Electron Vector                    *
 *    vector<Muon>     muons            -- Muon Vector                        *
 *    vector<Tau>      taus             -- Tau Vector                         *
 *    vector<Jet>      jets             -- Jet Vector                         *
 *    vector<MCParticle> MCparticles    -- MC Particle Vector                 *
 *                                                                            *
 * Protected Members of EventContainer class                                  *
 *    Int_t debugLevel                  -- Debug level                        *
 *    TEnv _config                      -- Used to read configuration         *
 *                                                                            *
 * Private Members of EventContainer class                                    *
 *    Double_t _targetTopMass           -- Target top mass                    *
 *    CollectionTree *_collectionTree   -- Collection tree                    *
 *    EventTree *_eventTree             -- Event tree                         *
 *    TruthTree *_thruthTree            -- Collection tree                    *
 *    Int_t _leptonSelect               -- Only process lep of certan flavor  *
 *    Double_t _normalization           -- Norm factor for all histograms     *
 *    Int_t _eventCount                 -- Location in event chain            *
 *    TString _sourceName               -- Name of source being read          *
 *    TString _sourceNumber             -- Number of source being read        *
 *    Double_t _globalEventWeight       -- Global Event Weight                *
 *    Double_t _treeEventWeight         -- Tree Event Weight                  *
 *    Double_t _outputEventWeight       -- Output Event Weight                *
 *                                                                            *
 *    Apply()                           -- Apply cuts and fill histograms     *
 *    GetCutName()                      -- Returns "CutNJets"                 *
 *                                                                            *
 * History                                                                    *
 *      19 Oct 2006 - Created by R. Schwienhorst for ATLAS                    *
 *      14 Nov 2006 - Modified by R. Schwienhorst - added tau, muon classes   *
 *      16 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#ifndef particles_h
#define particles_h

#include <vector>
#include <string>
#include <iostream>

// MakeClass output:
#include "single_top_analysis/CollectionTree.hpp"
#include "single_top_analysis/EventTree.hpp"
#include "single_top_analysis/TruthTree.hpp"

// Our own particles
#include "single_top_analysis/Electron.hpp"
#include "single_top_analysis/Muon.hpp"
#include "single_top_analysis/Tau.hpp"
#include "single_top_analysis/Jet.hpp"

// MC particles
#include "single_top_analysis/MCParticle.hpp"

// A global CutFlowTable to keep track of individual cuts
#include "single_top_analysis/CutFlowTable.hpp"

#include <TObjArray.h>
#include <TLorentzVector.h>
#include "TEnv.h"


class EventContainer
{
 public:

  //Default Constructor
  EventContainer();

  //Destructor
  ~EventContainer();

  // Get and Set the NAME of the source being run over
  inline void SetSourceName(const TString& name) { _sourceName = name; };
  inline TString GetSourceName() const { return _sourceName; };

  // Get and Set the NUMBER for the source being run over
  inline void SetSourceNumber(Int_t number) { _sourceNumber = number; };
  inline Int_t GetSourceNumber() const { return _sourceNumber; };

  // Get and Set the target top mass for top quark constraint in best jet algorithm
  inline void SetTargetTopMass(const Double_t& mass) { _targetTopMass = mass; };
  inline Double_t GetTargetTopMass() const { return _targetTopMass; };

  // Initialize this class 
  void Initialize(CollectionTree* cotr,EventTree* evtr, TruthTree* trtr);

  // Create TLorentz Vectors for the particles in the interaction
  Int_t ReadEvent();

  // Create final state particles and make cuts if necessary
  Bool_t CreateFinalState();

  // Reconstruct the top in several different ways from the final state objects
  void MakeTopQuarks();

  // Set up the particles for the next event
  Int_t GetNextEvent();

  // Initialize the event counter to get a specific event in the chain
  inline void SetEventCount(Int_t i) { _eventCount = i; };

  // Get and Set a specific flavor to process (0 to process all)
  inline void SetLeptonSelect(Int_t flavor) { _leptonSelect = flavor; };
  inline Int_t GetLeptonSelect() { return _leptonSelect; };

  // Set a normalization factor for all histograms
  inline void SetNormalization(double norm) { _normalization=norm; };

  // Get the collection tree
  inline CollectionTree* GetCollectionTree() const { return _collectionTree; };

  // Get the event tree
  inline EventTree* GetEventTree() const { return _eventTree; };

  // Get the truth tree
  inline TruthTree* GetTruthTree() const { return _truthTree; };
  
  // Get and Set Debug Level
  inline Int_t GetDebugLevel() const { return _debugLevel; };
  inline void SetDebugLevel(Int_t i) { _debugLevel=i; };

  // Set the global event weight
  inline void SetGlobalEventWeight(const Double_t& wt) { _globalEventWeight=wt; };
  // set the output event weight.
  inline void SetOutputEventWeight(const Double_t& wt) { _outputEventWeight=wt; };

  // Get the event weights for filling histograms
  inline Double_t GetEventWeight()       const { return _outputEventWeight; };
  inline Double_t GetOutputEventWeight() const { return _outputEventWeight; };
  inline Double_t GetTreeEventWeight()   const { return _treeEventWeight;   };
  inline Double_t GetGlobalEventWeight() const { return _globalEventWeight; };

  // Get the configuration 
  TEnv * GetConfig() {return &_config; };

  ////////////////////////////////////
  // Object Helpers:
  ////////////////////////////////////

  // Information for partons
  Int_t runNumber;              // run number from the root tree
  Int_t eventNumber;            // event number from the root tree

  // Vectors of Reconstructed particles
  std::vector<Electron>   electrons;
  std::vector<Muon>       muons;
  std::vector<Tau>        taus;
  std::vector<Jet>        jets;
  std::vector<MCParticle> MCparticles;    // The full list of all MC particles
  // ..... add more here

protected:
  
  // Debugging level
  Int_t _debugLevel;

  // Use TEnv to read configuration parameters
  TEnv _config;

private:
  
  // Target top mass used in the best jet algorithm
  Double_t _targetTopMass;

  // The classes that contain the information from the trees:
  CollectionTree *_collectionTree;
  EventTree      *_eventTree;
  TruthTree      *_truthTree;

  // Selects only certain lepton flavors
  Int_t _leptonSelect;     // = 11: electrons, =12: only muons, =0: all

  // Normalization factor for histograms
  Double_t _normalization; 

  // Counters to keep track of location in event chain
  Int_t _eventCount;

  // Info for source that we are reading (i.e. tb, tq, tW, tt_lepjets, tt_dilep, etc.
  TString _sourceName;    // Source Name
  Int_t   _sourceNumber;  // Source Number

  // Event Weight
  Double_t _globalEventWeight;         // Global event weight, unchanging event to event
  Double_t _treeEventWeight;           // Event weight read in from the tree
  Double_t _outputEventWeight;         // Output event weight passed to the histograms

  ////////////////////////////////////////////////////////////////////////////////
  // Integrate classes into the Root system
  // Must come at end of class definition
  ClassDef(EventContainer,0)
  ////////////////////////////////////////////////////////////////////////////////
};


#endif
