/******************************************************************************
 * Muon.hpp                                                                   *
 *                                                                            *
 * Store information about final state muons                                  *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of Muon class                                      *
 *    Muon()                             -- Default Constructor               *
 *    Muon()                             -- Parameterized Constructor         *
 *    Muon()                             -- Constructor with Particle         *
 *    ~Muon()                            -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetEtCone                          -- Set ET for EM layer 0             *
 *    GetEtCone                          -- Get ET for EM layer 0             *
 *    EtCone                             -- Get ET for EM layer 0             *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const Muon             *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of Muon class                                         *
 *    Double_t _EtCone                     -- CAL Energy in cone around muon  *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst                              *
 *      20 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#ifndef muon_h
#define muon_h

#include "single_top_analysis/Particle.hpp"
#include "single_top_analysis/EventTree.hpp"

class Muon: public Particle
{
public:

  // Default Constructor
  Muon();

  // Copy constructor from Muon
  Muon(const Muon& other);

  // Copy constructor from Particle
  Muon(const Particle& other);

  // Dectructor
  ~Muon();

  // Set all contents to their defaults
  inline void Clear() { Particle::Clear(); _EtCone = 0.0; };

  // Fill the muon from an EventTree
  void Fill(EventTree *evtr, Int_t iE);

  // Get and Set cone Et for this muon
  inline void SetEtCone(const double& val) { _EtCone=val; };
  inline double GetEtCone() const { return _EtCone; };
  inline double EtCone() const { return GetEtCone(); };

  // Overloaded operators:
  // +=
  Muon& operator+=(const Muon& other);
  // +
  Muon operator+(const Muon& other);
  // = Particle
  Muon& operator=(const Particle& other);
  // = Muon const
  Muon& operator=(const Muon& other);
  // = Muon non-const
  Muon& operator=(Muon& other);
    
private:
  // CAL Energy in a cone around the muon
  double _EtCone;   

  ////////////////////////////////////////////////////////////////////////////////
  // Integrate classes into the Root system
  // Must come at end of class definition
  ClassDef(Muon,0)
  ////////////////////////////////////////////////////////////////////////////////

};


#endif
