/******************************************************************************
 * AnalysisMain.cpp                                                           *
 *                                                                            *
 * This class controls the steering of the analysis.                          *
 *                                                                            *
 * Public Member Functions of AnalysisMain class                              *
 *    AnalysisMain()                    -- Default Constructo                 *
 *    ~AnalysisMain()                   -- Destructor                         *
 *    OpenHistogramFile()               -- Open file for output histograms    *
 *    WriteHistogramFile()              -- Write output histograms            *
 *    CloseHistogramFile()              -- Close output histogram file        *
 *    GetHistogramFileName()            -- Get file name                      *
 *    Loop()                            -- Loop over events                   *
 *    BookHistogram()                   -- Not needed for this class          *
 *    Apply()                           -- Not needed for this class          *
 *                                                                            *
 * Private Data Members of AnalysisMain class                                 *
 *     TFile *_histogramFile            -- Output histogram file              *
 *     string _histogramFileName        -- Output histogram file name         *
 *     Int_t _eventLimit               -- Number of events to loop over      *
 *     Bool_t _doSkim                   -- Flag for skimming                  *
 *     TFile *_skimFile                 -- Output skim file                   *
 *     string _skimFileName             -- Name of the skim file              *
 *     TTree* _skimCollectionTree       -- Collection tree in the skim file   *
 *     TTree* _skimEventTree            -- Event tree in the skim file        *
 *     TTree* _skimTruthTree            -- Truth tree in the skim file        *
 *                                                                            *
 *                                                                            *
 * History                                                                    *
 *     4 Jul 2005 - Created by R. Schwienhorst at D0                          *
 *     7 Nov 2006 - Modified by R. Schwienhorst for ATLAS                     *
 *     9 Nov 2006 - Modified by P. Ryan - cleanup and reorganized             *
 *****************************************************************************/

#include "single_top_analysis/AnalysisMain.hpp"
#include <iostream>
#include <string>
#include <fstream>
#include <iomanip>

#include <TStopwatch.h>
#include <TTreeIndex.h>
#include <TSystem.h>

#include "single_top_analysis/utils.hpp"

using namespace std;

// Integrate classes into the Root system
ClassImp(AnalysisMain)

/******************************************************************************
 * AnalysisMain::AnalysisMain()                                               *
 *                                                                            *
 * Default Constructor                                                        *
 * Initialize private data members of AnalysisMain                            *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None
 ******************************************************************************/
AnalysisMain::AnalysisMain(): CutListProcessor("cutter"),            // New Instance of CutListProcessor class
			      _histogramFile(NULL),                  // Output file points to NULL
			      _eventLimit(0),                        // Initialize max number of events processed
			      _histogramFileName("histograms.root"), // Initialize output file name
			      _skimFileName("NONE"),                 // Initialize skim output file name
			      _doSkim(kFALSE),                       // Don't skim by default
			      _skimCollectionTree(NULL),             // skimTree points to NULL
			      _skimEventTree(NULL),                  // skimTree points to NULL
			      _skimTruthTree(NULL)                   // skimTree points to NULL
{
  // cout << "AnalysisMain::AnalysisMain " << "Constructor called" << endl;
} // AnalysisMain::AnalysisMain()


/******************************************************************************
 * AnalysisMain::~AnalysisMain()                                              *
 *                                                                            *
 * Destructor                                                                 *
 * Do nothing except destruct                                                 *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
AnalysisMain::~AnalysisMain()
{
} // AnalysisMain::~AnalysisMain()


/******************************************************************************
 * AnalysisMain::ParseCmdLine()                                               *
 *                                                                            *
 * Parse the Command Line                                                     *
 *                                                                            *
 * Input:  Command Line Arguements and Chains of Events                       *
 * Output: Return 0 if successful 1 if unsuccessful (and print error)         *        
 ******************************************************************************/
Int_t AnalysisMain::ParseCmdLine(int argc, char **argv, TChain *chainCollection, TChain *chainEVO, TChain *chainTruth)
{

  /////////////////////////////////////////////////////////////////////////////////
  // Declare and Initialize Variables
  /////////////////////////////////////////////////////////////////////////////////
  // Offset from begining of chain
  Int_t evtOffSet = 0;

  // Variables for determining whether a list or single file is read in
  Int_t  fileListIndex   = 0;      // Parameter number of file-list name in cmd line arguments
  Int_t  fileSingleIndex = 0;      // Parameter number of file name in cmd line arguements
  Bool_t fileListFlag    = kFALSE; // Flag signifying use of file-list
  Bool_t fileSingleFlag  = kFALSE; // Flag signifying use of a single root file


  /////////////////////////////////////////////////////////////////////////////////
  // Check that we have valid chain pointers:
  /////////////////////////////////////////////////////////////////////////////////
  if(NULL == chainCollection) {
    cout << "<AnalysisMain::ParseCmdLine> " << "No valid input Collection Chain." << endl;
    return -1;
  }//if
  if(NULL == chainEVO) {
    cout << "<AnalysisMain::ParseCmdLine> " << "No valid input Event Chain." << endl;
    return -1;
  }//if
  if(NULL == chainTruth) {
    cout << "<AnalysisMain::ParseCmdLine> " << "No valid input Truth Chain." << endl;
    //cout << "<AnalysisMain::ParseCmdLine> " << "This is ok for now..." << endl;
    return -1;
  }//if

  /////////////////////////////////////////////////////////////////////////////////
  // Loop over arguements in command line 
  /////////////////////////////////////////////////////////////////////////////////
  cout << "<AnalysisMain::ParseCmdLine> " << "Parsing command line." << endl;

  // Print message if not enough arguements on command line
  if(argc < 3) {
    cout << "<AnalysisMain::ParseCmdLine> "<<"****************************************************************" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"* Program: AnalysisMain                                        *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                                                              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"* Purpose: analyze a parton level single top root file.        *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                                                              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"* Usage: AnalysisMain -inlist filename                         *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  You need to specify a root file or list of files            *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                                                              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -inlist file    - open a file with a list of root files.    *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -infile file    - load a single root file.                  *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                                                              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -config cfgfile - use this configuration file.              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                                                              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -source name    - name of the source we are processing.     *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                    Can also be passed through the list file. *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -sourcenum #    - number for the source we are processing.  *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                    Can also be passed through the list file. *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -num #          - number of events to process.              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -offSet #       - start processing not at 0 but here.       *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -hfile file     - filename for output histograms.           *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*  -skimfile file  - filename to write selected events to.     *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"*                                                              *" << endl;
    cout << "<AnalysisMain::ParseCmdLine> "<<"****************************************************************" << endl;
    return 1;
  } //if

  for (Int_t i = 1; i < argc; ++i) {
    //cout << "Command line parameter " << i << " is " << argv[i] << endl;

    // FLAG: List of root files
    if( !strcmp(argv[i],"-inlist") ) {
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing file-list name in cmd line" << endl;
	return 1;
      } //if argc < i+1
      fileListIndex = i+1;
      ++i;
      fileListFlag = kTRUE;
    } //if list of root files

    // FLAG: single root file
    else if( !strcmp(argv[i],"-infile") ) { 
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing input-file name in cmd line" << endl;
	return 1;
      }//if 
      fileSingleIndex = i + 1;
      ++i;
      fileSingleFlag = kTRUE;
    }//if single root file

    // FLAG: config file
    else if (!strcmp(argv[i],"-config")) {  
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing configuration file name in cmd line" << endl;
	return 1;
      }//if
      TString confFileName = argv[i+1];
      // Check if this input file exists!
      FILE *confFile;
      if(!(confFile = fopen(confFileName.Data(), "r"))) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Input configuration file " << confFileName.Data() 
	     << " does not exist!" << endl;
	return -1;
      } //if
      fclose(confFile);
      _config.ReadFile(confFileName.Data(), kEnvChange);
      ++i;
    }//if config file

    // FLAG: source name
    else if (!strcmp(argv[i],"-source")) { 
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing Value for source name -source!" << endl;
	return 1;
      }
      SetSourceName(argv[i+1]);
      ++i;
    } //if source name

    // FLAG: source number
    else if (!strcmp(argv[i],"-sourcenum")) { 
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing Value for source number -sourcenum!" << endl;
	return 1;
      }
      SetSourceNumber(atoi(argv[i+1]));
      ++i;
    } //if source number

    // FLAG: Max Number of Events
    else if (!strcmp(argv[i],"-num")) {  
      // Check if max number is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing Value for number of events -num!" << endl;
	return 1;
      } //if
      // Convert command line string into integer
      _eventLimit = atoi(argv[i+1]) + evtOffSet;
      ++i;
    } //if num events

    // FLAG: Event Offset
    else if (!strcmp(argv[i],"-offSet")) {  
      // Check if offset number is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing Value for number of events -offSet!" << endl;
	return 1;
      } //if
      // Convert command line string into integer
      evtOffSet=atoi(argv[i+1]);
      if(_eventLimit>0) _eventLimit+=evtOffSet;
      ++i;
    } //if event offset

    // FLAG: Output histogram file
    else if (!strcmp(argv[i],"-hfile")) {  
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing Value for histogram filename -hfile!" << endl;
	return 1;
      } //if
      _histogramFileName=argv[i+1];
      ++i;
    } //if output histogram file name

    // FLAG: Skim file
    else if (!strcmp(argv[i],"-skimfile")) {  
      // Check if file name parameter is in command line
      if (argc < i+1) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Missing Value for skim filename -skimfile!" << endl;
	return 1;
      } //if
      _doSkim = kTRUE;
      _skimFileName = argv[i+1];
      ++i;
    } //if skim file

  }//for loop over command line arguments
  
  /////////////////////////////////////////////////////////////////////////////////
  // Chain together root files
  /////////////////////////////////////////////////////////////////////////////////
  Int_t iChain;                    // Number of files in each line of chain file (after wildcard expansion)
  Bool_t doPathWarning = kTRUE;    // Prints warning message about directories

  if (fileListFlag) {
    // Use fstream
    ifstream chainFile(argv[fileListIndex]);
    Int_t chainLineCounter = 0;  // Total number of lines the file list
    Int_t chainFileCounter = 0;  // Total number of files in the file list (after wildcard expansion)
    
    if (chainFile) { // file with list exists
      string fileInChainName;
      
      cout << "<AnalysisMain::ParseCmdLine> " << "Chaining files listed in: ";
      cout << argv[fileListIndex] << endl;
      while (getline(chainFile, fileInChainName)) {
	// Check for comment lines
	if(fileInChainName.find("#") != string::npos) continue;

	// the line starting with Name: contains the name for this source.
	// just for informational purposes:
	if(fileInChainName.find("Name") != string::npos) {
	  string::size_type pos = fileInChainName.rfind(":");
	  string sname = fileInChainName.substr(pos+1);
	  cout<<"<AnalysisMain::ParseCmdLine> Name of this source: "<< fileInChainName<<endl;
	  SetSourceName(sname.c_str());
	  continue;
	}
	// The line starting with Number: contains the source number
	if(fileInChainName.find("Number") != string::npos) {
	  cout << "<AnalysisMain::ParseCmdLine> " << "Setting source number to: "<< fileInChainName.c_str() << endl;
	  string::size_type pos = fileInChainName.rfind(":");
	  string snu = fileInChainName.substr(pos+1);
	  SetSourceNumber(atoi(snu.c_str()));
	  continue;
	} //if
	cout << "<AnalysisMain::ParseCmdLine> " << "Reading root tree from " << fileInChainName.c_str() << endl;

	// Add files to chain for Collection, EVO, and Truth
	iChain = chainCollection -> Add(fileInChainName.c_str());
	chainEVO   -> Add(fileInChainName.c_str());
	chainTruth -> Add(fileInChainName.c_str());
	// Increment count of lines in list 
	if(iChain) ++chainLineCounter;
	// Update total number of files in list (accounting for wildcard expansion)
	chainFileCounter = chainFileCounter + iChain;
      } // while(getline)

      // Print number of lines and files read in from list
      cout << "<AnalysisMain::ParseCmdLine> " << chainLineCounter << " lines read from chain list." << endl;
      cout << "<AnalysisMain::ParseCmdLine> " << chainFileCounter << " files successfully chained from chain list." << endl;
      
      // Exit if no lines or files were read from list
      if ( (0 == chainLineCounter) || (0 == chainFileCounter) ) {
	cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Empty list in file " << argv[fileListIndex] << endl;
	return 1;
      } //if

    }//if(chainFile)
  } //if(fileListFlag) 

  /////////////////////////////////////////////////////////////////////////////////
  // Use Single Root File
  /////////////////////////////////////////////////////////////////////////////////
  else if (fileSingleFlag) { 
    // Add single file to chain
    cout << "<AnalysisMain::ParseCmdLine> " << "Reading in single file: " << argv[fileSingleIndex] << endl;
    iChain = chainCollection -> Add(argv[fileSingleIndex]);
    chainEVO -> Add(argv[fileSingleIndex]);
    chainTruth -> Add(argv[fileSingleIndex]);
    if (!iChain) {
      cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Chaining " << argv[fileSingleIndex];
      return 1;
    } //if
  } //if(fileSingleFlag)

  /////////////////////////////////////////////////////////////////////////////////
  // No chain or single file to input
  /////////////////////////////////////////////////////////////////////////////////
  else {
    cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: No chain or single file to input." << endl;
    return 1;
  } //else
  
  /////////////////////////////////////////////////////////////////////////////////
  // ???????????
  /////////////////////////////////////////////////////////////////////////////////
  // Create the root trees
  CollectionTree* treeCollection = new CollectionTree(chainCollection);
  EventTree*      treeEventTree  = new EventTree(chainEVO);
  TruthTree*      treeTruthTree  = new TruthTree(chainTruth);

  // Make EVO and Truth chains of Collection chain
  chainCollection -> AddFriend(chainEVO,"AliasForEV0");
  chainCollection -> AddFriend(chainTruth,"AliasForTruth0");

  // Enable only selected branches to improve speed
  // Disable Branches (chainCollection is small enought that disabling is probably not good idea)
  //   chainEVO   -> SetBranchStatus("*",0);
  //   chainTruth -> SetBranchStatus("*",0);
  // Now enable a select few branches
  //   chainEVO   -> SetBranchStatus("El_E",1);
  //   chainEVO   -> SetBranchStatus("El_N",1);
  //   chainEVO   -> SetBranchStatus("El_p_T",1);
  //   chainEVO   -> SetBranchStatus("El_eta",1);
  //   chainEVO   -> SetBranchStatus("El_phi",1);
  //   chainEVO   -> SetBranchStatus("El_EoverP",1);
  //   chainEVO   -> SetBranchStatus("El_charge",1);
  //   //chainEVO -> SetBranchStatus("",1);
  //   chainEVO   -> SetBranchStatus("PJet_N",1);


  // Open the output histogram file
  OpenHistogramFile(_histogramFileName);

  /////////////////////////////////////////////////////////////////////////////////
  // Parse the configuration file
  /////////////////////////////////////////////////////////////////////////////////

  // Debug level
  if(_config.Defined("DebugLevel")) {
    SetDebugLevel(_config.GetValue("DebugLevel",0));
    if(GetDebugLevel() > 0) cout << "<AnalysisMain::ParseCmdLine> " << "Parsing configuration file " << endl;
    if(GetDebugLevel() > 0) cout << "<AnalysisMain::ParseCmdLine> " << "Setting debug level to " <<GetDebugLevel() << endl;
  } //if

  // The target top mass used in some optimization (default: 175GeV)
  SetTargetTopMass(_config.GetValue("MTop",175.));
  if(GetDebugLevel() > 0) cout << "<AnalysisMain::ParseCmdLine> " << "Setting target top mass to " << GetTargetTopMass() << endl;

  // Include a file containing cuts
  if(_config.Defined("Include.CutsFile")) {
    TString cutsFile = _config.GetValue("Include.CutsFile","NONE");
    // Check if this input file exists!
    FILE *confFile;
    if(!(confFile = fopen(cutsFile.Data(), "r"))) {
      cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Cuts file "<<cutsFile.Data()<<" does not exist!"<<endl;
      return -1;
    } //if
    fclose(confFile);
    // The file exists, read it in:
    if(GetDebugLevel() > 0) cout << "<AnalysisMain::ParseCmdLine> " << "Reading cuts config file: " << cutsFile.Data() << endl;    
    _config.ReadFile(cutsFile.Data(),kEnvChange);
  } //if

  // Include a file containing weights for each source
  if(_config.Defined("Include.WeightsFile")) {
    TString weightsFile=_config.GetValue("Include.WeightsFile","NONE");
    // check if this input file exists!
    FILE *weightFile;
    if(!(weightFile = fopen(weightsFile.Data(), "r"))) {
      cout << "<AnalysisMain::ParseCmdLine> " << "ERROR: Weights file " << weightsFile.Data()<<" does not exist!" << endl;
      return -1;
    } //if
    fclose(weightFile);
    // The file exists, read it in:
    if(GetDebugLevel() > 0) cout << "<AnalysisMain::ParseCmdLine> " << "Reading weights config file: " << weightsFile.Data() << endl;    
    _config.ReadFile(weightsFile.Data(), kEnvChange);
  } //if
  
  /////////////////////////////////////////////////////////////////////////////////
  // If we want to skim
  /////////////////////////////////////////////////////////////////////////////////
  if(_doSkim && _skimFileName!="NONE") {
    // Open Skim File
    OpenSkimFile(_skimFileName);
    // Create clones of chains
    _skimCollectionTree = (TTree*)chainCollection -> CloneTree(0);
    _skimEventTree      = (TTree*)chainEVO        -> CloneTree(0);
    _skimTruthTree      = (TTree*)chainTruth      -> CloneTree(0);
  } //if

  /////////////////////////////////////////////////////////////////////////////////
  // The chains were read in successfully. Now initialize the analysis classes
  /////////////////////////////////////////////////////////////////////////////////
  EventContainer::Initialize(treeCollection,treeEventTree,treeTruthTree);
  EventContainer::SetEventCount(evtOffSet);

  return 0;
} //AnalysisMain::ParseCmdLine

/******************************************************************************
 * void AnalysisMain::Loop()                                                  *
 *                                                                            *
 * Loops over Events                                                          *
 * Main Action of Program                                                     *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
 void AnalysisMain::Loop()
{
  // Add histogram to _histogramFile directory in memory
  SetDirectory (_histogramFile);
  // Book Histograms
  CutListProcessor::BookHistogram();

  // Set flag if an event limit is specified (_eventLimit read from cmd line)
  Bool_t eventLimitFlag = (_eventLimit > 0);

  // Start the stopwatch
  TStopwatch stopwatch;
  stopwatch.Start();

  // Get Event count which is determined from position in chain
  Int_t eventInChain = GetNextEvent();

  // Initialize Event count which is determined by looping over events
  Int_t eventCounter = 0;
  
  // Initialize flags for writing events
  Bool_t writeThisEvent    = kFALSE;
  Bool_t firstEventWritten = kFALSE;

  // Loop over events in chain until (1) Out of events or (2) Have reached limit specified on command line
  //cout << "<AnalysisMain::Loop> " <<  "Beginning loop over events, _numEvent is "<<_eventLimit << endl;
  cout << "<AnalysisMain::Loop> " << endl;
  while( eventInChain != -1 && (!eventLimitFlag || eventInChain < _eventLimit) ) {  
    eventCounter++;
    if( 0 == (eventInChain%10000) ) cout << "<AnalysisMain::Loop> " << "Processing event " << eventInChain << endl;

    // Fill the histograms
    writeThisEvent = CutListProcessor::Apply(this);

    // Fill Trees for Skim Events
    if(_doSkim && writeThisEvent) {
      if (!firstEventWritten) {
	firstEventWritten = kTRUE;
	//add_skim_branches();
      } //if
      if(NULL != _skimCollectionTree) _skimCollectionTree -> Fill();
      if(NULL != _skimEventTree)      _skimEventTree      -> Fill();
      if(NULL != _skimTruthTree)      _skimTruthTree      -> Fill();
    } //if

    // Read the next event in the chain
    // cout << "<AnalysisMain::Loop> " << "Getting next event in chain" << endl;
    eventInChain = GetNextEvent();
  } // while

  cout << "<AnalysisMain::Loop> " << "Processed " << eventCounter << " events." << endl;

  //
  // Print everything!
  //
  
  PrintCutDetails();

  //
  // Stop the timer and print out timing statistics
  //
  stopwatch.Stop();
  Double_t timeReal = stopwatch.RealTime();
  Double_t timeEvent = timeReal / static_cast<Double_t>(eventCounter);

  cout << "<AnalysisMain::Loop> " << endl;
  cout << "<AnalysisMain::Loop> " <<"           ------------------------------- " << endl;
  cout << "<AnalysisMain::Loop> " <<"          |         TIMING INFO           |"  << endl;
  cout << "<AnalysisMain::Loop> " <<"           ------------------------------- " << endl;
  cout << "<AnalysisMain::Loop> " <<"          |  Real time:      " << setw(10) << setprecision(4) << timeReal      << " s" << " |"  << endl;
  cout << "<AnalysisMain::Loop> " <<"          |  Time per event: " << setw(10) << setprecision(4) << timeEvent     << " s" << " |"  << endl;
  cout << "<AnalysisMain::Loop> " <<"          |  Events/second:  " << setw(10) << setprecision(4) << 1.0/timeEvent << "  " << " |"  << endl;
  cout << "<AnalysisMain::Loop> " <<"           ------------------------------- " << endl;
  cout << "<AnalysisMain::Loop> " << endl;

  // Write and Close histogram file
  WriteHistogramFile();
  CloseHistogramFile();
  
  // If applicable Write and Close skim file
  if(_doSkim) {
    WriteSkimFile();
    CloseSkimFile();
  } //if

} // void AnalysisMain::Loop()
