/******************************************************************************
 * MCElectron.cpp                                                             *
 *                                                                            *
 * Container for a MCElectron                                                 *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of MCElectron class                                *
 *    MCElectron()                       -- Default Constructor               *
 *    MCElectron()                       -- Parameterized Constructor         *
 *    MCElectron()                       -- Constructor with Particle         *
 *    ~MCElectron()                      -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetEoverP                          -- Set E over P                      *
 *    GetEoverP                          -- Get E over P                      *
 *    EoverP                             -- Get E over P                      *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const MCElectron       *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of MCElectron class                                   *
 *    Double_t _EoverP                   -- CAL Energy in cone around muon    *
 *                                                                            *
 * History                                                                    *
 *      15 Nov 2006 - Created by R. Schwienhorst                              *
 *      20 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#include "single_top_analysis/MCElectron.hpp"
#include <iostream>
#include <string>
#include <fstream>


using namespace std;

// Integrate classes into the Root system
ClassImp(MCElectron)

/******************************************************************************
 * MCElectron::MCElectron()                                                   *
 *                                                                            *
 * Default Constructor                                                        *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
MCElectron::MCElectron() : MCParticle::MCParticle(), _EoverP(0.0)
{

} //MCElectron()

/******************************************************************************
 * MCElectron::~MCElectron()                                                  *
 *                                                                            *
 * Destructor                                                                 *  
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
MCElectron::~MCElectron()
{

} //~MCElectron()


/******************************************************************************
 * MCElectron::MCElectron(const MCElectron& other)                            *
 *                                                                            *
 * Copy constructor MCElectron const                                          *
 *                                                                            *
 * Input:  MCElectron                                                         *
 * Output: None                                                               *
 ******************************************************************************/
MCElectron::MCElectron(const MCElectron& other): MCParticle(other),_EoverP(other.EoverP())
{

}//MCElectron()

/******************************************************************************
 * MCElectron::MCElectron(const MCElectron& other)                            *
 *                                                                            *
 * Copy constructor MCParticle const                                          *
 *                                                                            *
 * Input:  MCElectron                                                         *
 * Output: None                                                               *
 ******************************************************************************/
MCElectron::MCElectron(const MCParticle& other): MCParticle(other),_EoverP(0.0)
{

} //MCElectron

/******************************************************************************
 * MCElectron& MCElectron::operator+=(const MCElectron& other)                *
 *                                                                            *
 * Overload +=                                                                *
 *                                                                            *
 * Input:  MCElectron                                                         *
 * Output: MCElectron                                                         *
 ******************************************************************************/
MCElectron& MCElectron::operator+=(const MCElectron& other)
{
  
  MCParticle::operator+=(other);
  // also add EoverPs
  SetEoverP(GetEoverP() + other.GetEoverP());
  return *this;
} //+=

/******************************************************************************
 * MCElectron& MCElectron::operator+(const MCElectron& other)                 *
 *                                                                            *
 * Overload +                                                                 *
 *                                                                            *
 * Input:  MCElectron                                                         *
 * Output: MCElectron                                                         *
 ******************************************************************************/
MCElectron MCElectron::operator+(const MCElectron& other)
{

  MCParticle mcParticleTemp = MCParticle::operator+(other);
  MCElectron mcElectronTemp = mcParticleTemp;
  
  // I don't know how to add the EoverP for two different MCelectrons.
  // For now, add the energies and add the momenta and then divide.
  Double_t EoP  = 0.0;
  Double_t E1   = E(), 
  Double_t E2   = other.E();
  Double_t Ptot = 0.0;
 
  if(GetEoverP() != 0.0) Ptot = E()/GetEoverP();
  else E1 = 0.0;

  if(other.EoverP() != 0.0) Ptot += other.E()/other.GetEoverP();
  else E2 = 0.0;

  if(Ptot == 0.0) _EoverP = 0.0;
  else _EoverP = (E1 + E2) / Ptot;

  mcElectronTemp.SetEoverP( _EoverP );

  // Return the Temp particle
  return mcElectronTemp;
} //+

/******************************************************************************
 * MCElectron& MCElectron::operator=(const Muon& other)                       *
 *                                                                            *
 * Overload = MCParticle                                                      *
 *                                                                            *
 * Input:  MCParticle                                                         *
 * Output: MCElectron                                                         *
 ******************************************************************************/
MCElectron& MCElectron::operator=(const MCParticle& other)
{
  
  MCParticle::operator=(other);
  SetEoverP(0.0);
  return *this;
} // MCParticle

/******************************************************************************
 * MCElectron& MCElectron::operator=(const MCElectron& other)                 *
 *                                                                            *
 * Overload = const MCElectron                                                *
 *                                                                            *
 * Input:  MCElectron                                                         *
 * Output: MCElectron                                                         *
 ******************************************************************************/
MCElectron& MCElectron::operator=(const MCElectron& other)
{
  MCParticle::operator=(other);
  SetEoverP(other.GetEoverP());
  return *this;
} //MCElectron

/******************************************************************************
 * MCElectron& MCElectron::operator=(MCElectron& other)                       *
 *                                                                            *
 * Overload = non-const MCElectron                                            *
 *                                                                            *
 * Input:  MCElectron                                                         *
 * Output: MCElectron                                                         *
 ******************************************************************************/
MCElectron& MCElectron::operator=(MCElectron& other)
{
    MCParticle::operator=(other);
  SetEoverP(other.GetEoverP());
  return *this;
} //MCElectron()

/******************************************************************************         
 * void MCElectron::Fill(TruthTree *truthTree, Int_t iE)                      *         
 *                                                                            *         
 * Fill MCElectron vector from tree                                           *         
 *                                                                            *         
 * Input:  Event Tree                                                         *         
 * Output: None                                                               *         
 ******************************************************************************/
void MCElectron::Fill(TruthTree *truthTree,int iE)
{

    Double_t elPt     = truthTree -> El_p_T     -> operator[](iE)/1000.0;
    Double_t elEta    = truthTree -> El_eta     -> operator[](iE);
    Double_t elPhi    = truthTree -> El_phi     -> operator[](iE);
    Double_t elE      = truthTree -> El_E       -> operator[](iE);
    Int_t elPdgId     = truthTree -> El_pdgId   -> operator[](iE);
    Int_t elStat      = truthTree -> El_status  -> operator[](iE);
    Int_t elBc        = truthTree -> El_BarCode -> operator[](iE);
    _EoverP           = truthTree -> El_EoverP  -> operator[](iE);
    Double_t elCharge = truthTree -> El_charge  -> operator[](iE);

    SetPtEtaPhiE(elPt, elEta, elPhi, elE);
    SetCharge(elCharge);
    SetPdgId(elPdgId);
    SetStatus(elStat);
    SetBarCode(elBc);

} //Fill()


