/******************************************************************************
 * Muon.cpp                                                                   *
 *                                                                            *
 * Store information about final state muons                                  *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of Muon class                                      *
 *    Muon()                             -- Default Constructor               *
 *    Muon()                             -- Parameterized Constructor         *
 *    Muon()                             -- Constructor with Particle         *
 *    ~Muon()                            -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetEtCone                          -- Set ET for EM layer 0             *
 *    GetEtCone                          -- Set ET for EM layer 0             *
 *    EtCone                             -- Set ET for EM layer 0             *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const Muon             *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of Muon class                                         *
 *    Double_t _EtCone                     -- CAL Energy in cone around muon  *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst                              *
 *      20 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/

#include "single_top_analysis/Muon.hpp"
#include <iostream>
#include <string>
#include <fstream>


using namespace std;

// Integrate classes into the Root system
ClassImp(Muon)

/******************************************************************************
 * Muon::Muon()                                                               *
 *                                                                            *
 * Default Constructor                                                        *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Muon::Muon() : Particle::Particle(),_EtCone(0.0)
{
} //Muon

/******************************************************************************
 * Muon::~Muon()                                                              *
 *                                                                            *
 * Destructor                                                                 *  
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Muon::~Muon()
{
  //cout<<"In muon destructor"<<endl;
} //~Muon

/******************************************************************************
 * Muon::Muon(const Muon& other)                                              *
 *                                                                            *
 * Copy constructor Muon const                                                *
 *                                                                            *
 * Input:  Muon                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Muon::Muon(const Muon& other): Particle(other),_EtCone(other.GetEtCone())
{
} //Muon()

/******************************************************************************
 * Muon::Muon(const Muon& other)                                              *
 *                                                                            *
 * Copy constructor Particle const                                            *
 *                                                                            *
 * Input:  Muon                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Muon::Muon(const Particle& other): Particle(other),_EtCone(0.0)
{
} //Muon

/******************************************************************************
 * Muon& Muon::operator+=(const Muon& other)                                  *
 *                                                                            *
 * Overload +=                                                                *
 *                                                                            *
 * Input:  Muon                                                               *
 * Output: Muon                                                               *
 ******************************************************************************/
Muon& Muon::operator+=(const Muon& other)
{
  
  Particle::operator+=(other);
  // Also add EtCones
  SetEtCone(EtCone() + other.GetEtCone());
  return *this;
}

/******************************************************************************
 * Muon& Muon::operator+(const Muon& other)                                   *
 *                                                                            *
 * Overload +                                                                 *
 *                                                                            *
 * Input:  Muon                                                               *
 * Output: Muon                                                               *
 ******************************************************************************/
Muon Muon::operator+(const Muon& other)
{

  Particle particleTemp = Particle::operator+(other);
  Muon muonTemp = particleTemp;

  // I don't know how to add the EtCone for two different muons.
  // for now simply add them
  _EtCone += other.GetEtCone();

  muonTemp.SetEtCone( _EtCone );

  // Return muonTemp
  return muonTemp;
} //+

/******************************************************************************
 * Muon& Muon::operator=(const Muon& other)                                   *
 *                                                                            *
 * Overload = Particle                                                        *
 *                                                                            *
 * Input:  Particle                                                           *
 * Output: Muon                                                               *
 ******************************************************************************/
Muon& Muon::operator=(const Particle& other)
{
  Particle::operator=(other);
  SetEtCone(0.0);
  return *this;
} //= Particle

/******************************************************************************
 * Muon& Muon::operator=(const Muon& other)                                   *
 *                                                                            *
 * Overload = const Muon                                                      *
 *                                                                            *
 * Input:  Muon                                                               *
 * Output: Muon                                                               *
 ******************************************************************************/
Muon& Muon::operator=(const Muon& other)
{
  Particle::operator=(other);
  SetEtCone(other.GetEtCone());
  return *this;
} //= const muon

/******************************************************************************
 * Muon& Muon::operator=(const Muon& other)                                   *
 *                                                                            *
 * Overload = non-const Muon                                                  *
 *                                                                            *
 * Input:  Muon                                                               *
 * Output: Muon                                                               *
 ******************************************************************************/
Muon& Muon::operator=(Muon& other)
{
  Particle::operator=(other);
  SetEtCone(other.GetEtCone());
  return *this;
} //= non-const muon

/******************************************************************************         
 * void Muon::Fill(EventTree *evtr, Int_t iE)                                 *         
 *                                                                            *         
 * Fill Muon vector from tree                                                 *         
 *                                                                            *         
 * Input:  Event Tree                                                         *         
 * Output: None                                                               *         
 ******************************************************************************/
void Muon::Fill(EventTree *evtr,int iE)
{

    Double_t muPt     = evtr -> Mu_p_T    -> operator[](iE) / 1000.0;
    Double_t muEta    = evtr -> Mu_eta    -> operator[](iE);
    Double_t muPhi    = evtr -> Mu_phi    -> operator[](iE);
    Double_t muE      = evtr -> Mu_E      -> operator[](iE);
    _EtCone           = evtr -> Mu_etcone -> operator[](iE) / 1000.0;
    Double_t muCharge = evtr -> Mu_charge -> operator[](iE);

    SetPtEtaPhiE(muPt, muEta, muPhi, muE);
    SetCharge(muCharge);

} //Fill()


