/******************************************************************************
 * Tau.hpp                                                                    *
 *                                                                            *
 * Store information about final state Taus                                   *
 *                                                                            *
 * Derived from Particle class                                                *
 *                                                                            *
 * Public Member Functions of Tau class                                       *
 *    Tau()                              -- Default Constructor               *
 *    Tau()                              -- Parameterized Constructor         *
 *    Tau()                              -- Constructor with Particle         *
 *    ~Tau()                             -- Destructor                        *
 *    Clear()                            -- Set contents to default           *
 *    Fill                               -- Fill electron from event tree     *
 *    SetSetNumTracks                    -- Set Num Tracks                    *
 *    GetSetNumTracks                    -- Set Num Tracks                    *
 *    NumTracks                          -- Set Num Tracks                    *
 *    operator+=                         -- Overload +=                       *
 *    operator+                          -- Overload +                        *
 *    operator=                          -- Overload = Particle               *
 *    operator=                          -- Overload = const Tau              *
 *    operator=                          -- Overload =                        *
 *                                                                            *
 * Private Data Members of Tau class                                          *
 *    Double_t _NumTracks                -- CAL Energy in cone around Tau     *
 *                                                                            *
 * History                                                                    *
 *      14 Nov 2006 - Created by R. Schwienhorst                              *
 *      20 Nov 2006 - Modified by P. Ryan - cleanup and reorganized           *
 *****************************************************************************/
#include "single_top_analysis/Tau.hpp"
#include <iostream>
#include <string>
#include <fstream>


using namespace std;

// Integrate classes into the Root system
ClassImp(Tau)
/******************************************************************************
 * Tau::Tau()                                                                 *
 *                                                                            *
 * Default Constructor                                                        *
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Tau::Tau() : Particle::Particle(),_NumTracks(0.0)
{

} //Tau()

/******************************************************************************
 * Tau::~Tau()                                                              *
 *                                                                            *
 * Destructor                                                                 *  
 *                                                                            *
 * Input:  None                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Tau::~Tau()
{

} //~Tau()


/******************************************************************************
 * Tau::Tau(const Tau& other)                                              *
 *                                                                            *
 * Copy constructor Tau const                                                *
 *                                                                            *
 * Input:  Tau                                                               *
 * Output: None                                                               *
 ******************************************************************************/
Tau::Tau(const Tau& other): Particle(other),_NumTracks(other.GetNumTracks())
{

} //Tau()

/******************************************************************************
 * Tau::Tau(const Tau& other)                                                 *
 *                                                                            *
 * Copy constructor Particle const                                            *
 *                                                                            *
 * Input:  Tau                                                                *
 * Output: None                                                               *
 ******************************************************************************/
Tau::Tau(const Particle& other): Particle(other),_NumTracks(0.0)
{

} //Tau() particle

/******************************************************************************
 * Tau& Tau::operator+=(const Tau& other)                                     *
 *                                                                            *
 * Overload +=                                                                *
 *                                                                            *
 * Input:  Tau                                                                *
 * Output: Tau                                                                *
 ******************************************************************************/
Tau& Tau::operator+=(const Tau& other)
{
  Particle::operator+=(other);
  // Also add NumTracks
  SetNumTracks(GetNumTracks() + other.GetNumTracks());
  return *this;
} //+=

/******************************************************************************
 * Tau& Tau::operator+(const Tau& other)                                      *
 *                                                                            *
 * Overload +                                                                 *
 *                                                                            *
 * Input:  Tau                                                                *
 * Output: Tau                                                                *
 ******************************************************************************/
Tau Tau::operator+(const Tau& other)
{

  Particle particleTemp = Particle::operator+(other);
  Tau tauTemp = particleTemp;
  
  // I don't know how to add the NumTracks for two different taus.
  // For now, add them
  _NumTracks += other.GetNumTracks();
  tauTemp.SetNumTracks( _NumTracks );

  // Return the temp particle
  return tauTemp;
} //+

/******************************************************************************
 * Tau& Tau::operator=(const Particle& other)                                 *
 *                                                                            *
 * Overload = Particle                                                        *
 *                                                                            *
 * Input:  Particle                                                           *
 * Output: Tau                                                                *
 ******************************************************************************/
Tau& Tau::operator=(const Particle& other)
{
  Particle::operator=(other);
  SetNumTracks(0.0);
  return *this;
} // = Particle

/******************************************************************************
 * Tau& Tau::operator=(const Tau& other)                                      *
 *                                                                            *
 * Overload = const Tau                                                       *
 *                                                                            *
 * Input:  Tau                                                                *
 * Output: Tau                                                                *
 ******************************************************************************/
Tau& Tau::operator=(const Tau& other)
{
  
  Particle::operator=(other);
  SetNumTracks(other.GetNumTracks());
  return *this;
} //= const tau

/******************************************************************************
 * Tau& Tau::operator=(const Tau& other)                                      *
 *                                                                            *
 * Overload = non-const Tau                                                   *
 *                                                                            *
 * Input:  Tau                                                                *
 * Output: Tau                                                                *
 ******************************************************************************/
Tau& Tau::operator=(Tau& other)
{
  
  Particle::operator=(other);
  SetNumTracks(other.GetNumTracks());
  return *this;
} //= tau non-const

/******************************************************************************         
 * void Tau::Fill(EventTree *evtr, Int_t iE)                                 *         
 *                                                                            *         
 * Fill Tau vector from tree                                                 *         
 *                                                                            *         
 * Input:  Event Tree                                                         *         
 * Output: None                                                               *         
 ******************************************************************************/
void Tau::Fill(EventTree *evtr,int iE)
{

    Double_t tauPt     = evtr -> Tau_p_T      -> operator[](iE) / 1000.0;
    Double_t tauEta    = evtr -> Tau_eta      -> operator[](iE);
    Double_t tauPhi    = evtr -> Tau_phi      -> operator[](iE);
    Double_t tauE      = evtr -> Tau_E        -> operator[](iE);
    _NumTracks         = evtr -> Tau_numTrack -> operator[](iE);
    Double_t tauCharge = evtr -> Tau_charge   -> operator[](iE);

    SetPtEtaPhiE(tauPt, tauEta, tauPhi, tauE);
    SetCharge(tauCharge);

} //Fill()


